@@ -1,8 +1,8 @@
 import os
 
-from PySide2 import QtCore
-from PySide2.QtGui import *
-from PySide2.QtWidgets import *
+from qtpy import QtCore
+from qtpy.QtGui import *
+from qtpy.QtWidgets import *
 
 
 class TrialListWidget(QWidget):
